@@ -65,7 +65,11 @@
 #ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
-#endif /* !SALONEINSTLIB */
+#include "numlib.h"
+#else /* SALONEINSTLIB */
+#include "sa_config.h"
+#include "numsup.h"
+#endif /* SALONEINSTLIB */
 #include "xspect.h"
 #include "insttypes.h"
 #include "icoms.h"
@@ -146,7 +150,6 @@ ss_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout) {
 	ss_ctt sobrc[7]  = { ss_ctt_SetBaud9600, ss_ctt_SetBaud19200, ss_ctt_SetBaud57600,
 	                     ss_ctt_SetBaud2400, ss_ctt_SetBaud1200,  ss_ctt_SetBaud600,
 	                     ss_ctt_SetBaud300 };
-	char *fcc;
 	ss_ctt fcc1;
 	ss_hst fcc2;
 	long etime;
@@ -343,7 +346,7 @@ ss_init_coms(inst *pp, int port, baud_rate br, flow_control fc, double tout) {
 	return inst_ok;
 }
 
-/* Set the capabilities values for the current mode */
+/* Set the capabilities values for the type of instrument */
 static void ss_determine_capabilities(ss *p) {
 
 	/* Set the capabilities mask */
@@ -421,7 +424,6 @@ ss_init_inst(inst *pp) {
 			return rv;
 
 		if (p->verb) {
-			char buf[1000]; 
 			char dn[19];		/* Device name */
 			unsigned int sn;	/* Serial number */
 			char pn[9];			/* Part number */
@@ -455,7 +457,6 @@ ss_init_inst(inst *pp) {
 		return rv;
 
 	if (p->verb) {
-		char buf[1000]; 
 		char dn[19];		/* device name */
 		ss_dnot dno;		/* device number */
 		char pn[9];			/* part number */
@@ -509,6 +510,11 @@ struct _inst *pp) {
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_holdrel)
 		rv = ss_do_ReleasePaper(p);
 	return rv;
@@ -522,6 +528,16 @@ struct _inst *pp) {
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_holdrel)
 		rv = ss_do_HoldPaper(p);
 	return rv;
@@ -535,6 +551,11 @@ struct _inst *pp) {
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_locate) {
 		rv = ss_do_SetDeviceOffline(p);
 		p->offline = 1;
@@ -552,6 +573,11 @@ double x, double y
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_position)
 		if ((rv = ss_do_MoveAbsolut(p, measure ? ss_rt_SensorRef : ss_rt_SightRef, x, y)) != inst_ok)
 			return rv;
@@ -570,6 +596,11 @@ double *x, double *y) {
 	ss_rt rr;
 	ss_zkt zk;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_position)
 		if ((rv = ss_do_OutputActualPosition(p, ss_rt_SightRef, &rr, x, y, &zk)) != inst_ok)
 			return rv;
@@ -585,6 +616,11 @@ struct _inst *pp) {
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_position) {
 		rv = ss_do_SetDeviceOnline(p);
 		p->offline = 0;
@@ -600,6 +636,11 @@ struct _inst *pp) {
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->cap2 & inst2_xy_position) {
 		ss_do_SetDeviceOnline(p);	/* Put the device online */
 		ss_do_MoveUp(p);			/* Raise the sensor */
@@ -632,7 +673,11 @@ ipatch *vals) { 		/* Pointer to array of values */
 	int pass, step, patch;
 	int tries, tc;			/* Total read tries */
 	int fstep = 0;			/* NZ if step is fast & quiet direction */
-	int pisrow;				/* patches in a step direction serpentine row */
+
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
 
 	if (p->itype != instSpectroScan
 	 && p->itype != instSpectroScanT)
@@ -889,6 +934,11 @@ ipatch *val) {		/* Pointer to instrument patch value */
 	double col[3], spec[36];
 	int i;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (!val)
 		return inst_internal_error;
 
@@ -1062,7 +1112,7 @@ ipatch *val) {		/* Pointer to instrument patch value */
 	/* Using filter compensation */
 	/* This isn't applicable to emulated transmission mode, because */
 	/* the filter will be calibrated out in the illuminant measurement. */
-	} else if (p->compen |= 0) {
+	} else if (p->compen != 0) {
 		ss_cst rct;
 		ss_st rst;		/* Return Spectrum Type (Reflectance/Density) */
 		ss_rvt rvf;		/* Return Reference Valid Flag */
@@ -1119,14 +1169,12 @@ ipatch *val) {		/* Pointer to instrument patch value */
 				XYZ[j] += obsv[tix][j][i] * spec[i];
 
 		if ((p->mode & inst_mode_illum_mask) == inst_mode_emission) {
-			/* The CIE maximum spectral luminence efficiency is 683 lumens per watt, */
-			/* which is the constant applied to sumation over 1nm from 360 to 830nm, */
-			/* so this needs to be scaled by the sumation over 5nm from 380 to 830, */
-			/* a factor of 10.683/106.86 * 683. */
+			/* Emission XYZ seems to be Luminous Watts, so */
+			/* convert to cd/m^2 */
 			val->aXYZ_v = 1;
-			val->aXYZ[0] = XYZ[0] * 683.226;
-			val->aXYZ[1] = XYZ[1] * 683.226;
-			val->aXYZ[2] = XYZ[2] * 683.226;
+			val->aXYZ[0] = XYZ[0] * 683.002;
+			val->aXYZ[1] = XYZ[1] * 683.002;
+			val->aXYZ[2] = XYZ[2] * 683.002;
 		} else {
 			val->XYZ_v = 1;
 			val->XYZ[0] = XYZ[0];
@@ -1173,6 +1221,7 @@ ipatch *val) {		/* Pointer to instrument patch value */
 			val->sp.spec_wl_short = 380;
 			val->sp.spec_wl_long = 730;
 			if ((p->mode & inst_mode_illum_mask) == inst_mode_emission) {
+				/* Spectral data is in mW/nm/m^2 interpolated to 10nm spacing */
 				val->sp.norm = 1.0;
 				for (i = 0; i < val->sp.spec_n; i++)
 					val->sp.spec[i] = (double)spec[i];
@@ -1194,6 +1243,11 @@ ipatch *val) {		/* Pointer to instrument patch value */
 inst_cal_type ss_needs_calibration(inst *pp) {
 	ss *p = (ss *)pp;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	if (p->need_w_cal && p->noautocalib == 0) {
 		return inst_calt_ref_white;
 
@@ -1461,6 +1515,11 @@ char id[CALIDLEN]		/* Condition identifier (ie. white reference ID) */
 ) {
 	ss *p = (ss *)pp;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	return ss_calibrate_imp(p, calt, calc, id);
 }
 
@@ -1474,6 +1533,11 @@ char *filtername
 ) {
 	ss *p = (ss *)pp;
 	
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 #ifndef SALONEINSTLIB
 	if (filtername == NULL) {
 		p->compen = 0;
@@ -1680,10 +1744,16 @@ static inst_code
 ss_set_mode(inst *pp, inst_mode m) {
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
-
-	int cap = pp->capabilities(pp);
+	int cap;
 	inst_mode mm;		/* Measurement mode */
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
+	cap = pp->capabilities(pp);
+
 	/* The measurement mode portion of the mode */
 	mm = m & inst_mode_measurement_mask;
 
@@ -1757,6 +1827,11 @@ inst_status_type m,	/* Requested status type */
 	ss *p = (ss *)pp;
 	inst_code rv = inst_ok;
 
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
+
 	/* Return the filter */
 	if (m == inst_stat_get_filter) {
 		inst_opt_filter *filt;
@@ -1810,11 +1885,10 @@ ss_set_opt_mode(inst *pp, inst_opt_mode m, ...)
 {
 	ss *p = (ss *)pp;
 
-	/* Ignore these modes - not applicable, but be nice. */
-	if (m == inst_opt_disp_crt
-	 || m == inst_opt_disp_lcd) {
-		return inst_ok;
-	}
+	if (!p->gotcoms)
+		return inst_no_coms;
+	if (!p->inited)
+		return inst_no_init;
 
 	if (m == inst_opt_noautocalib) {
 		p->noautocalib = 1;
@@ -1851,6 +1925,8 @@ ss_set_opt_mode(inst *pp, inst_opt_mode m, ...)
 			case inst_opt_filter_Custom:
 				p->filt = ss_aft_CustomFilter;
 				return inst_ok;
+			default:
+				break;
 		}
 		return inst_unsupported;
 	}
@@ -1890,7 +1966,7 @@ ss_del(inst *pp) {
 }
 
 /* Constructor */
-extern ss *new_ss(icoms *icom, int debug, int verb) {
+extern ss *new_ss(icoms *icom, instType itype, int debug, int verb) {
 	ss *p;
 	if ((p = (ss *)calloc(sizeof(ss),1)) == NULL)
 		error("ss: malloc failed!");
@@ -1928,7 +2004,7 @@ extern ss *new_ss(icoms *icom, int debug, int verb) {
 	p->del          	= ss_del;
 
 	/* Init state */
-	p->itype = instUnknown;						/* Unknown until initialised */
+	p->itype = itype;							/* Preliminary */
 	p->cap = inst_unknown;						/* Unknown until initialised */
 	p->mode = inst_mode_unknown;				/* Not in a known mode yet */
 	p->nextmode = inst_mode_unknown;			/* Not in a known mode yet */
@@ -1956,5 +2032,7 @@ extern ss *new_ss(icoms *icom, int debug, int verb) {
 	p->sby = 200.0;
 #endif
 
+	ss_determine_capabilities(p);
+
 	return p;
 }
